<?php

namespace Drupal\inotherwords\Plugin\Field\FieldFormatter;

use Drupal\text\Plugin\Field\FieldFormatter\TextDefaultFormatter;
use function Agaric\OxfordComma\oxford_comma_list;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'inotherwords string' formatter.
 *
 * It applies to multiple value "string" items.
 *
 * @FieldFormatter(
 *   id = "inotherwords_string",
 *   label = @Translation("In other words: String"),
 *   description = @Translation("Provide format settings including series comma such that given Apple Banana Cantaloupe as selected options content shows Apple, Banana, and Cantaloupe."),
 *   field_types = {
 *     "string",
 *   }
 * )
 */
class InotherwordsStringFormatter extends TextDefaultFormatter {
  use InotherwordsFormatterHelperTrait;

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return self::commonDefaultSettings() + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    return $this->commonSettingsFormElements() + parent::settingsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $summary[] = t('Strings separated by "@series_join" with "@series_final_join" at the end, prefaced by "@text_before" and followed by "@text_after".',
      [
        '@series_join' => $this->getSetting('series_join'),
        '@series_final_join' => $this->getSetting('series_final_join'),
        '@text_before' => $this->getSetting('text_before'),
        '@text_after' => $this->getSetting('text_after'),
      ]
    );
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    // If items have no text format, force plain text.
    foreach ($items as $delta => $item) {
      $items[$delta]->format = $item->format ?? 'plain_text';
    }

    $elements = parent::viewElements($items, $langcode);

    // Not sure if we'd ever be given zero elements, but if so, give 'em back.
    if (!$elements) {
      return $elements;
    }

    $single = count($elements) === 1;

    $series_final_join = $this->getSetting('series_final_join');
    $series_join = $this->getSetting('series_join');
    $text_before = $this->getSetting('text_before');
    $text_after = $this->getSetting('text_after');

    $oxford_comma_settings = [
      'join' => ['#markup' => $series_join],
      'final_join' => ['#markup' => $series_final_join ],
      'oxford' => $this->getSetting('oxford_comma'),
    ];

    $inotherwords = oxford_comma_list($elements, $oxford_comma_settings);

    $theme_suggest = $this->additionalThemeSuggestions($items);

    // Needless nesting needed for FormatterBase::view() to add field wrappers.
    $elements = [];
    $elements[0] = [
      '#theme' => 'inotherwords_series_wrapped__string' . $theme_suggest,
      '#text_before' => self::processPlural($text_before, $single),
      '#items' => $inotherwords,
      '#text_after' => self::processPlural($text_after, $single),
    ];
    return $elements;
  }

}
